#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "matrix.h"
#include "pnm.h"

/** Calculate a distance map 
 * using the chamfer 3-4, this is an window
 * of size 3x3 which with local calculations
 * aproximate (withing a error of 8%) to the 
 * Euclidian distance
 * **/
void chamfer34( MATRIX *IN )
{
	int i, j, l, k, m;
	byte *ptrin, *ptrout;
	int val1[6] = { 4, 3, 4,   3, 0, 255};
	int val2[6] = { 4, 3, 4, 255, 0, 3  };
	int r[6];

	/** limpio del borde de la imagen  **/
    ptrin = (byte *) IN->ptr[0];
    for( j=0; j<IN->cols; j++ ) *ptrin++ = 0;
    ptrin = (byte *) IN->ptr[ IN->rows - 1 ];
    for( j=0; j<IN->cols; j++ ) *ptrin++ = 0;

    for( i=1; i<IN->rows-1; i++ ){
        ptrin = (byte *) IN->ptr[i];
        *ptrin = 0;
        ptrin = (byte *) IN->ptr[i] + IN->cols - 1;
        *ptrin = 0;
    }
    /** termin de limpiar **/


	/** the result is stored in the same input image **/
	/** Forward mask**/
	for( i=0; i<=IN->rows - 3; i++ ) {
		ptrout = (byte *) IN->ptr[i+1] + 1;
		for( j=0; j<=IN->cols - 3; j++ ) {
			for( l=0; l<2; l++ ) { 
				ptrin = (byte *) IN->ptr[i + l] + j;
				for( k=0; k<3; k++ ) {
					m = 3*l + k;
				   	r[m] = *ptrin +  val1[m]; 
					ptrin++;
				}
			}
			m = r[0];
			for( k=1; k<5; k++ ) 
				if ( r[k] < m ) m = r[k]; 

			if ( m > 255 ) m = 255;
			*ptrout++ = m;	
		}
	}
	/** Backward mask **/
	for( i=IN->rows-1; i>=2; i-- ) {
		ptrout = (byte *) IN->ptr[i-1] + IN->cols - 2;
		for( j=IN->cols-1; j>=2; j-- ) {
			for( l=1; l>=0; l-- ) { 
				ptrin = (byte *) IN->ptr[i - l] + j - 2;
				for( k=0; k<3; k++ ) {
					m = 3*l + k;
				   	r[m] = *ptrin++ +  val2[m]; 
				}
			}
			m = r[0];
			for( k=1; k<=5; k++ ) 
				if ( r[k] < m ) m = r[k]; 

			if ( m > 255 ) m = 255;

			*ptrout-- = m;	
		}
	}
}

int main( int argc, char *argv[] )
{
	MATRIX *IMG;
	char path[80];

	if ( argc < 3 ) {
		printf("Uso: %s img_entrada.pbm img_salida.pbm\n", argv[0]);
		exit(1);
	}

	strncpy( path, argv[1], 80 );
	printf ( "Se lee la imagen de entrada %s\n", path );
	IMG = read_pnm( path );
	if ( IMG == NULL ){
		fprintf ( stderr, "Error en el archivo de imagen de entrada %s\n", path );
		return( -1 );
	}
	
	chamfer34( IMG );


	strncpy( path, argv[2], 80 );
	IMG->type = 5;
	write_pnm( IMG, path );

	matrix_free(IMG);

	return ( 0 );
}

