/**
	Program to apply a filter in the hartley domain

**/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "matrix.h"
#include "tiff.h"
#include "utils.h"

int do_work( char *namein, int filtertype, int r ) 
{
	MATRIX *IN;
	MATRIX *FIN;
	MATRIX *DHTIMG;
	MATRIX *OUT;
	FILEINFO finfo;
	int i, j, signo;
	byte *iptr;
	float *fptr, *fptr2;
	

	IN = read_tiff( &finfo, namein ); 
	if ( IN==NULL ) {
		fprintf( stderr, "ERROR reading %s file\n", namein );
		return -1;
	}
	FIN = matrix_allocate( IN->rows, IN->cols, sizeof(float) );

	/** We copy the readed image to the new float matrix **/
	/** The variable signo is for to move the HT a half of a period **/
	signo = -1;
	for ( i=0; i<IN->rows; i++ ) {
		iptr = (byte *)IN->ptr[i];
		fptr = (float *)FIN->ptr[i];
		for ( j=0; j<IN->cols; j++ ) {
			signo *= -1;
			*fptr++ = *iptr++ * signo;
		}
		signo *= -1;
	}

	/** Now we calculate the DHT **/
	init_dht( IN->rows );
	DHTIMG = image_dht ( FIN );

	/** We apply the filter **/
	hartleyfilter( (float **)DHTIMG->ptr, IN->rows , IN->cols, r, filtertype );

	/** For testing, we apply the inverse transform **/
	OUT = image_dht ( DHTIMG );

	/** Reverse the signs **/
	signo = -1;
	for ( i=0; i<IN->rows; i++ ) {
		fptr = (float *)OUT->ptr[i];
		for ( j=0; j<IN->cols; j++ ) {
			signo *= -1;
			*fptr++ *= signo;
		}
		signo *= -1;
	}

	/** We convert the result to a 8 bit matrix and then we store
		the resulted image **/
	rescale( (float **)OUT->ptr, (byte **)IN->ptr, IN->rows, IN->cols );

	if ( write_tiff( IN, "filtered.tif" ) != GOOD_WRITE )
		return -1;

	free_dht();
	matrix_free( IN );
	matrix_free( FIN );
	matrix_free( DHTIMG );
	matrix_free( OUT );
}

int main( int argc, char *argv[] )
{
	char namein[32];
	int type, radius;

	if ( argc < 4 ) { 
		fprintf( stderr, "Use: %s image.tif type radius\n", argv[0] );
		fprintf( stderr, "\t type=0 Ideal Lowpass\n" );
		fprintf( stderr, "\t type=1 Ideal Highpass\n" );
		fprintf( stderr, "\t type=2 Raised cosine Lowpass\n" );
		fprintf( stderr, "\t type=3 Raised cosine Highpass\n" );
		fprintf( stderr, "\t type=4 Gauss Lowpass\n" );
		fprintf( stderr, "\t type=5 Gauss Highpass\n" );
		fprintf( stderr, "\t type=6 Fermi Lowpass\n" );
		fprintf( stderr, "\t type=7 Fermi Highpass\n" );
		return -1;
	}

	strncpy( namein, argv[1], 32 );
	type = atoi( argv[2] );
	radius = atoi( argv[3] );

	if ( do_work( namein, type, radius ) >= 0 ) {
		fprintf( stderr, "OK!!!\n" );
		return 0;
	}
	else {
		fprintf( stderr, "ERROR: Something is wrong!!!\n" );
		exit(1);
	}
	return 0;
}
