/**
	Filtering operations using the FFT

	Program to apply a filter in the frequency domain
	using the FFT functions 

	Fraga. Jun 12, 2001
**/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "matrix.h"
#include "tiff.h"
#include "fft.h"
#include "utils.h"

int do_work( char *namein, int filtertype, float r, int orden ) 
{
	MATRIX *IN;
	MATRIX *FFTIMG;
	FILEINFO finfo;
	int i, j, signo;
	byte *iptr;
	float *fptr;
	

	IN = read_tiff( &finfo, namein ); 
	if ( IN==NULL ) {
		fprintf( stderr, "ERROR reading %s file\n", namein );
		return -1;
	}
	FFTIMG = matrix_allocate( IN->rows, IN->cols + 2, sizeof(float) );
	
	if ( r <= 0.5 ) r *= IN->rows;

	/** We copy the readed image to the new float matrix **/
	/** The variable signo is for move the FT a half of a period **/
	signo = -1;
	for ( i=0; i<IN->rows; i++ ) {
		iptr = (byte *)IN->ptr[i];
		fptr = (float *)FFTIMG->ptr[i];
		for ( j=0; j<IN->cols; j++ ) {
			signo *= -1;
			*fptr++ = *iptr++ * signo;
	//		*fptr++ = *iptr++;
		}
		signo *= -1;
	}

	/** Now we calculate the FFT **/
	image_fft ( (float **)FFTIMG->ptr, IN->rows , IN->cols, DIRECT );

	/** We apply the filter **/
	filter( (float **)FFTIMG->ptr, IN->rows , IN->cols, r, filtertype, orden );

	/** For testing, we apply the inverse transform **/
	image_fft ( (float **)FFTIMG->ptr, IN->rows , IN->cols, INVERSE );

	/** Reverse the signs **/
	signo = -1;
	for ( i=0; i<IN->rows; i++ ) {
		fptr = (float *)FFTIMG->ptr[i];
		for ( j=0; j<IN->cols; j++ ) {
			signo *= -1;
			*fptr++ *= signo;
		}
		signo *= -1;
	}

	/** We put the result in a 8 bit matrix and then we store
		the resulted image. This must be the equal to the original image **/
	rescale( (float **)FFTIMG->ptr, (byte **)IN->ptr, IN->rows, IN->cols );
	
	if ( write_tiff( IN, "filtered.tif" ) != GOOD_WRITE )
		return -1;

	matrix_free( IN );
	matrix_free( FFTIMG );

	return 0;
}

int main( int argc, char *argv[] )
{
	char namein[32];
	int type;
	float radius;
	int orden=0;

	if ( argc < 4 ) { 
		fprintf( stderr, "Use: %s image.tif type radius\n", argv[0] );
		fprintf( stderr, "\t type=0 Ideal LP,      type=1 IdealHP\n" );
		fprintf( stderr, "\t type=2 Raised cos LP, type=3 Raised cos HP\n" );
		fprintf( stderr, "\t type=4 Gauss LP,      type=5 Gauss HP\n" );
		fprintf( stderr, "\t type=6 Fermi LP,      type=7 Fermi HP\n" );
		fprintf( stderr, "\t type=8 Butterworth LP\n" );
		return -1;
	}

	strncpy( namein, argv[1], 32 );
	type = atoi( argv[2] );
	radius = atof( argv[3] );

	if ( type == 8 ) {
		orden = atoi( argv[4] );
	}
	if ( do_work( namein, type, radius, orden ) >= 0 ) {
		fprintf( stderr, "OK!!!\n" );
		return 0;
	}
	else {
		fprintf( stderr, "ERROR: Something is wrong!!!\n" );
		exit(1);
	}
	return 0;
}
