/**
	One idea for one paper

	Input image must be in PPM format

	Fraga.  Enero 21, 2009
**/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "matrix.h"
#include "pnm.h"

int do_work( char *namein, char *nameout, int c ) 
{
	MATRIX *RGBIN, *BOUT;
	int i, j;
	RGB *iptr;
	byte *optr;
	

	RGBIN = read_pnm( namein ); 
	if ( RGBIN==NULL ) {
		fprintf( stderr, "ERROR reading %s file\n", namein );
		return -1;
	}
	if ( RGBIN->type != 6 ) {
		fprintf( stderr, "ERROR Bad format. Image must be in RGB, ppm format\n");
		return -1;
	}

	/** Escribo solo la imagen del canal azul **/

	BOUT = matrix_allocate( RGBIN->rows, RGBIN->cols, 1 );

	if ( c == 0 ) {
		for( i=0; i<RGBIN->rows; i++ ) {
			iptr = (RGB *) RGBIN->ptr[i];
			optr = (byte *) BOUT->ptr[i];
			for( j=0; j<RGBIN->cols; j++ ) *optr++ = (*iptr++).R;;
		}
	}
	else if ( c == 1 ) {
		for( i=0; i<RGBIN->rows; i++ ) {
			iptr = (RGB *) RGBIN->ptr[i];
			optr = (byte *) BOUT->ptr[i];
			for( j=0; j<RGBIN->cols; j++ ) *optr++ = (*iptr++).G;;
		}
	}
	else {
		for( i=0; i<RGBIN->rows; i++ ) {
			iptr = (RGB *) RGBIN->ptr[i];
			optr = (byte *) BOUT->ptr[i];
			for( j=0; j<RGBIN->cols; j++ ) *optr++ = (*iptr++).B;;
		}
	}

	if ( BOUT == NULL ) {
		fprintf( stderr, "ERROR doing histogram equalization\n");
	 	return -1;
	}
	BOUT->type=5;
	if ( write_pnm( BOUT, nameout ) != 0 ) {
		fprintf( stderr, "ERROR writing output image\n");
		return -1;
	}

	matrix_free( RGBIN );
	matrix_free( BOUT );

	return 0;
}

int main( int argc, char *argv[] )
{
	char namein[32], nameout[32];
	int val;

	if ( argc < 4 ) { 
		fprintf( stderr, "Use: %s input_image.pgm output_image_name.pgm channel[0-R, 1-G, 2-B]\n", argv[0] );
		return -1;
	}

	strncpy( namein, argv[1], 32 );
	strncpy( nameout, argv[2], 32 );
	val = strtol( argv[3], (char **) NULL, 10);
	if ( val < 0 || val > 2 ){ 
		fprintf( stderr, "Error: channel must be 0 for R, 1 for G, or 2 for B\n" );
		return 1;	
	}

	if ( !do_work( namein, nameout, val ) ) {
		fprintf( stderr, "OK!!!\n" );
	}
	else {
		fprintf( stderr, "ERROR: Something is wrong!!!\n" );
		return(1);
	}
	return 0;
}
