/**
	Program to calculate the histogram of a image 

	Input image must be in PBM format

	Fraga. Jun 4, 2001
**/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "matrix.h"
#include "utils.h"
#include "pnm.h"

/** 
	Output: Return histogram in a MATRIX form: an image of
	        size 1x256 
	Input: Image file name
**/
MATRIX * histogram_from_file( char *namein ) 
{
	MATRIX *IN;
	MATRIX *HIST;
	int i, j;
	byte *iptr;
	int *histo;
	
	IN = read_pnm( namein ); 
	if ( IN==NULL ) {
		fprintf( stderr, "ERROR reading %s file\n", namein );
		return NULL;
	}
	if ( IN->type != 5 ) {
		fprintf( stderr, "ERROR Bad format. Image must be in 256 grey tones\n");
		return NULL;
	}

	HIST = histogram( IN ); 

	matrix_free( IN );

	return( HIST );
}

int main( int argc, char *argv[] )
{
	MATRIX *HISTO;
	char namein[32];

	if ( argc < 2 ) { 
		fprintf( stderr, "Use: %s image.pnm\n", argv[0] );
		return -1;
	}

	strncpy( namein, argv[1], 32 );

	if ( (HISTO = histogram_from_file( namein )) != NULL ) {
		print_histogram( HISTO );
		fprintf( stderr, "OK!!!\n" );
		return 0;
	}
	else {
		fprintf( stderr, "ERROR: Something is wrong!!!\n" );
		exit(1);
	}
	return 0;
}
