/** Creamos una imagen de prueba **/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "matrix.h"
#include "pnm.h"

void rotate( MATRIX *I, MATRIX *O, float ang )
{
	byte **po, **pi;
	int i, j;
	float s, c, cx, cy;
	float xi, yi, x, y;
	int  a, b, xint, yint, n;

	s = sin( ang*3.1416/180.0 );
	c = cos( ang*3.1416/180.0 );

	/**  De los pixels de la imagen rotada, calculamos cada pixel de la imagen original **/

	po = (byte **) O->ptr;
	pi = (byte **) I->ptr;

	if ( I->rows > I-> cols ) { 
		n = I->cols;
		a = 0;
		b = (I->rows - I->cols)/2;
	}
	else {
		n = I->rows;
		a = (I->cols - I->rows)/2;
		b = 0;
	}
	cx = a + (n - 1 ) /2.0;
	cy = b + (n - 1 ) /2.0;


	xi = (a-cx)*c + (b-cy)*s;
	yi =-(a-cx)*s + (b-cy)*c;

		
	for( i=0; i<n; i++ ){
		x = xi;
		y = yi;
		for( j=0; j<n; j++ ){
			xint = (int)(x + cx + 0.5);
			yint = (int)(y + cy + 0.5);

			if ( xint >=0 && xint < I->cols ) 
				if ( yint >=0 && yint < I->rows ) 
					po[ i + b ][ j + a  ] = pi[yint][xint];			

			x += c;
			y -= s;
		}
		xi += s;
		yi += c;
	}
}

int main( int argc, char *argv[] )
{
	MATRIX *I, *O;
	unsigned char *p;
	int i, j;
	float ang;

	if ( argc != 3 ) {
		fprintf( stderr, "Args: image_name angle\n" );
		return 1;
	}

	I = read_pnm( argv[1] );
	if ( I == NULL ) {
		fprintf( stderr, "Error reading %s file\n", argv[1] );
		return 1;
	}
	ang = strtod( argv[2], (char **) NULL);

	/** Reservar memoria para la imagen **/
	O = matrix_allocate( I->rows, I->cols, 1 );

	/** Limpio la imagen de salida **/
	for( i=0; i<O->rows; i++ ){
		p = (byte *)O->ptr[i];
		for( j=0; j<O->cols; j++ ){
			*p++ = 0x00;
		}
	}

	rotate( I, O, ang );

	if ( !write_pnm( O, "rotated.pgm" ) ) {
		fprintf( stderr, "Ok!\n" );
	}
	else
		fprintf( stderr, "Error!\n" );

	matrix_free( I );
	matrix_free( O );

	return 0;
}
