#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <string.h>
#include "tiff.h"

#ifndef MASKTABLE
unsigned char masktable[8]={0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01};
#define MASKTABLE 1
#endif

short int numbertype;


void setdefaults(FILEINFO *fi)
{
	int i;
	fi->width = 0;
	fi->depth = 0;
	fi->bits = 0;
	fi->photometric = 0;
	fi->planarconfig = 1;
	fi->samples = 1;
	fi->compression = 1;
	fi->offset = 0L;
	/** Paleta por defecto a 256 tonos de grises **/
	for(i=0; i<256; ++i)
		memset(fi->palette + (i*TAM),i,TAM);
}


/** Obtiene un word (2 bytes) de un archivo TIFF **/
unsigned int fgetword( FILE *fp )   
{
	int a, b;
	a = fgetc(fp);
	b = fgetc(fp);
	if (numbertype == II) return( a + (b << 8) );
	else return( (a << 8) + b );
}

/** Obtiene un int long (4 bytes ) de un archivo TIFF **/
unsigned long fgetlong( FILE *fp )
{
	int a, b, c, d;
	a = fgetc( fp );
	b = fgetc( fp );
	c = fgetc( fp );
	d = fgetc( fp );
	if (numbertype == II)
		return( (unsigned long) a  +
			((unsigned long) b  << 8) +
			((unsigned long) c  << 16) +
			((unsigned long) d  << 24)  );
	else
		return( ((unsigned long) a << 24) +
			((unsigned long) b << 16) +
			((unsigned long) c << 8) +
			((unsigned long) d ) );
}


/** Lee una lnea de PackBits comprimidos **/
int readline(char *p, FILE *fp, int bytes, FILEINFO *fi)
{
	int c,i,n=0;

	if (fi->compression == 1) {
		n = fread(p,1,bytes,fp);
		if ( fi -> photometric == 0 )
			for ( i=0; i< bytes; i++ ) p[i] = ~p[i];
		return( n );
	}
	else if (fi->compression == 0x8005)
		{
			do {
				c = fgetc(fp);
				if (c & 0x0080) {
					if (c != 0x0080)
					{
						i = ((~c) & 0x00ff)+2;
						c = fgetc(fp);
						if ( fi -> photometric == 0 )
							while(i--) p[n++] = (char)~c;
						else
                        	while(i--) p[n++] = (char)c;
					}
				}
				else {
					i = (c & 0x00ff)+1;
                    if ( fi -> photometric == 0 )
                    	while(i--) p[n++] = (char)~fgetc(fp);
					else
						while(i--) p[n++] = (char) fgetc(fp);
				}
			} while(n < bytes);
		return(n);
	}
	else return(0);
}


void decodetag( FILEINFO *fi, FILE *fp)
{
	long length, offset, pos;
	int tag, type, i;

	tag = fgetword(fp);
	type = fgetword(fp);
	if(type == TIFFlong)
	{
		length = fgetlong(fp);
		offset = fgetlong(fp);
	}
	else {
		if (numbertype == II) {
			length = (unsigned long)fgetword(fp);
			fgetword(fp);
			offset = (unsigned long)fgetword(fp);
			fgetword(fp);
		}
		else {
			fgetword(fp);
			length = (unsigned long)fgetword(fp);
			if ( type == TIFFshort && length <= 1L ) {
				offset = (unsigned long)fgetword(fp);
				fgetword(fp);
			}
			else {
				fgetword(fp);
				offset = (unsigned long)fgetword(fp);
			}
		}
	}
	switch(tag)
	{
		case SubfileType     : break;
		case ImageWidth      : fi->width = (unsigned int)offset;
			break;
		case ImageLength     : fi->depth = (unsigned int)offset;
			break;
		case RowsPerStrip    : if(type == TIFFlong) fi->rowsperstrip = (unsigned int)offset;
			else fi->rowsperstrip = (unsigned int) ( offset & 0xffffL );
			break;
		case StripOffsets    : if(type == TIFFlong) fi->offset = offset;
			else fi->offset = offset & 0xffffL;
			fi->count = (unsigned int)length;
			break;
		case StripByteCounts : if(type == TIFFlong) fi->bytecount = offset;
			else fi->bytecount = offset & 0xffffL;
			break;
		case SamplesPerPixel : fi->samples = (unsigned int)offset;
			break;
		case BitsPerSample   :
			if(length > 1L) {
				pos = ftell(fp);
				fseek(fp,offset,SEEK_SET);
				fi->bitspersample = fgetword(fp);
				fseek(fp,pos,SEEK_SET);
			}
			else fi->bitspersample = (int)offset;
			break;
		case PlanarConfiguration : fi->planarconfig = (int)offset;
			break;
		case Compression     : fi->compression = (int)offset;
			break;
		case Group3Options   : break;
		case Group4Options   : break;
		case FillOrder       : break;
		case Threshholding   : break;
		case CellWidth       : break;
		case CellLength      : break;
		case MinSampleValue  : break;
		case MaxSampleValue  : break;
		case PhotometricInterp :
			fi -> photometric = (unsigned int) offset;
			break;
		case GrayResponseUnit  : break;
		case GrayResponseCurve : break;
		case ColorResponseUnit : break;
		case ColorResponseCurve : break;
		case XResolution     : break;
		case YResolution     : break;
		case ResolutionUnit  : break;
		case Orientation     : break;
		case DocumentName    : break;
		case PageName        : break;
		case XPosition       : break;
		case YPosition       : break;
		case PageNumber      : break;
		case ImageDescription : break;
		case Make            : break;
		case Model           : break;
		case FreeOffsets     : break;
		case FreeByteCounts  : break;
		case ColorMap        : pos = ftell(fp);
				fseek(fp,offset,SEEK_SET);
				for(i=0; i < (1 << fi->bitspersample); ++i) {
					if (i >= 256) break;
					fi->palette[i*TAM + ROJO] = (char)(fgetword(fp) >> 8);
				}
				for (i=0; i<(1<<fi->bitspersample); ++i) {
					if (i >= 256) break;
					fi->palette[i*TAM + VERDE] = (char)(fgetword(fp) >> 8);
				}
				for (i=0; i<(1<<fi->bitspersample); ++i) {
					if (i >= 256) break;
					fi->palette[i*TAM + AZUL] = (char) (fgetword(fp) >> 8);
				}
				fseek(fp,pos,SEEK_SET);
				break;
		}
}


void convertline(char *dest,char *source, FILEINFO *fi)
{
	unsigned short int i;

	switch(fi->bits)
	{
	
		case 8  :
			memcpy( dest, source, fi->widthbuffer );
			break;
		case 4  : 
			for (i=0; i< fi->widthbuffer; ) {
				dest[i++] = (*source>>4) & 0x0f;
				dest[i++] = *source & 0x0f;
				++source;
			}
			break;
		case 1  :
			for (i=0; i< fi->widthbuffer; i++) {
				if ( source[ i>>3 ] & masktable[ i & 0x0007 ] )  
					dest[i] = 0xff;
				else dest[i] = 0x00;
			}
			break;			

	}
}


MATRIX * read_tiff( FILEINFO *fi, char *path)
{
	FILE *fp;
	MATRIX *IN;
	char *p, *pr;
	unsigned long l;
	unsigned int k;
	int i,j,n;

	if((fp = fopen(path,"r")) == NULL) return( NULL );

	numbertype = fgetword(fp);        /* Obtiene numero de tipos */
	if(numbertype == II || numbertype == MM)
	{                                 /* Ve si es un archivo TIFF real */
		fgetword(fp);                   /* Tras la version */
		while((l=fgetlong(fp)) != 0L)   /* Busca a traves de directorios de imagenes */
		{
			fseek(fp,l,SEEK_SET);
			setdefaults(fi);              /* Establece valores por default */
			n = fgetword(fp);             /* Decodifica todos los tags */
			for (i=0; i<n; ++i)
				decodetag(fi,fp);
			l=ftell(fp);                  /* Graba la posicion del archivo */
			if ( fi->photometric > 1 ) return( NULL ); /* Solo acepta tonos de gris */
			fi -> bits = fi->samples * fi->bitspersample;
			switch( fi->bits) {
				case 1:
					fi->bytes=pixels2bytes(fi->width);
					fi->widthbuffer = fi->width;
					break;
				case 4:
					if ( fi->width%2 ) fi->bytes = (fi -> width >> 1) + 1;
					else	fi->bytes= fi->width >> 1;
					fi->widthbuffer = fi->width;
					break;
				case 8:
					if (fi->compression == 1)
						fi->bytes = fi->width;
					else if (fi->compression == 0x8005)
						fi->bytes = pixels2bytes(fi->width)*fi->bits;
					fi->widthbuffer = fi->width;
					break;
				default: /** Error **/
					fprintf(stderr, "Tipo de imagen (en bits por pixel) no soportado\n");
					fclose( fp );
					return( NULL );
			}

 			/** Localiza memoria para la imagen leida **/
			IN = matrix_allocate( fi->depth, fi->widthbuffer, 1);
			if( IN == NULL) {
				fclose( fp );
				return( NULL );
			}
			/** Invariablemente el tipo sera a 1 byte por pixel,
				para trabajar con los pixeles como unsigned char **/
			IN->type = 5;

			if (fi->width != 0 && fi->depth != 0 && fi->offset != 0L)
			{
				if ((p = (char *)malloc(fi->bytes)) == NULL) {
					fclose( fp );
					return( NULL );
				}
				if ((pr = (char *)malloc(fi->widthbuffer)) == NULL){
					free(p);
					fclose( fp );
					return( NULL );
				}
				if (fi->count == 1L)
				{
					fseek(fp,fi->offset,SEEK_SET);
					for(i=0; i < (int)fi->depth; ++i)
					{
						if ( readline( p, fp, fi->bytes, fi) != (int)(fi->bytes) )
						{
							free(pr); free(p);
							matrix_free(IN);
							fclose( fp );
							return( NULL );
						}
						convertline(pr,p,fi);
						memcpy( (char *)IN->ptr[i], pr, IN->cols);

					}
				}
				else {
					for (k=0; k<(fi->count); ++k)
					{
						fseek(fp,fi->offset+(k*sizeof(long)),SEEK_SET);
						fseek(fp,fgetlong(fp),SEEK_SET);
						for (j=0; j<(int)fi->rowsperstrip; ++j)
						{
							if (((k*fi->rowsperstrip)+j) >= (unsigned long)(fi->depth))
							break;
							if (readline(p, fp, fi->bytes, fi) != (int)fi->bytes)
							{
								free(pr); free(p);
								matrix_free(IN);
								fclose( fp );
								return( NULL );
							}
							convertline( pr, p, fi);
							memcpy( (char *)IN->ptr[k*fi->rowsperstrip+j], pr, IN->cols);

						}
					}
				}
				free(pr); free(p);
				fclose( fp );
				return( IN );
			 	/** Es posible en TIFF que se almacenen
				varias imagenes en un solo archivo pero no se contempla,
				solo traera la primera imagen, en dado caso **/
			/**	fseek( fp, l, SEEK_SET ); **/ 
			}
		} 
		return(NULL);
	}
	else return(NULL);
}



