/**
	Program to calculate the cosine transform of
	an input image

	Fraga, Jul 30, 2001

**/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "matrix.h"
#include "pnm.h"
#include "utils.h"

int do_work( char *namein ) 
{
	MATRIX *IN, *M, *OUT, *T;
	int i, j;
	byte *iptr;
	float *fptr, *fptr2;

#ifdef TEST
	float kk[8] = {1,1,1,1,0,0,0,0};
	float r[8], r2[8];

	init_dct(8);
	printf( "# Sequence     Direct DCT    Inverse DCT\n" );

	dct( kk, r, 8, 0);
	dct( r, r2, 8, 1);

	for ( i=0; i<8; i++ ) printf( "%d %e %e %e\n", i,  kk[i], r[i], r2[i] );
	free_dct( );
#endif
	
	IN = read_pnm( namein ); 
	if ( IN==NULL ) {
		fprintf( stderr, "ERROR reading %s file\n", namein );
		return -1;
	}
	if ( IN->type != 5 ) {
		fprintf( stderr, "ERROR Bad format. Image must be in 256 grey tones\n");
		return -1;
	}
	if ( IN->rows != IN->cols ) {
		fprintf( stderr, "ERROR: Input image must be a square\n");
		return -1;
	}
	M = matrix_allocate( IN->rows, IN->cols, sizeof(float) );
	if ( M==NULL ) {
		fprintf( stderr, "ERROR allocating memory for output image\n" );
		return -1;
	}
	/** We copy the input image to a float image buffer **/
	for ( i=0; i<IN->rows; i++ ) {
		iptr = (byte *) IN->ptr[i];
		fptr = (float *) M->ptr[i];
		for ( j=0; j<IN->cols; j++ ) *fptr++ = *iptr++;
	}
	/** We initialize table and buffers for calculate the DCT **/
	init_dct( IN->rows );

	OUT = image_dct( M, 0 );

	for ( i=0; i<IN->rows; i++ ) {
		fptr = (float *) OUT->ptr[i];
		fptr2 = (float *) M->ptr[i];
		for ( j=0; j<IN->cols; j++ ) *fptr2++ = log( fabs(*fptr++) + 1.0 );
	}
	rescale( (float **)M->ptr, (byte **)IN->ptr, IN->rows, IN->cols );
	IN->type=5;
	if ( write_pnm( IN, "logcosine.pgm" ) != 0 )  
		return -1;

	T = image_dct( OUT, 1 );
	free_dct( );


	for ( i=0; i<IN->rows; i++ ) {
		iptr = (byte *) IN->ptr[i];
		fptr = (float *) T->ptr[i];
		for ( j=0; j<IN->cols; j++ ) *iptr++ = (byte) ( *fptr++ + 0.2 );
	}

	IN->type=5;
	if ( write_pnm( IN, "test.pgm" ) != 0 )  
		return -1;

	matrix_free( T );
	matrix_free( IN );
	matrix_free( M );
	matrix_free( OUT );

	return 0;
}

int main( int argc, char *argv[] )
{
	char namein[32];

	if ( argc < 2 ) { 
		fprintf( stderr, "Use: %s image.pgm\n", argv[0] );
		return -1;
	}

	strncpy( namein, argv[1], 32 );

	if ( do_work( namein ) >= 0 ) {
		fprintf( stderr, "OK!!!\n" );
		return 0;
	}
	else {
		fprintf( stderr, "ERROR: Something is wrong!!!\n" );
		exit(1);
	}
	return 0;
}
